%------ CREATE TOTAL TORQUE MATRIX T ------------------------------------
%
%   The following code calculates a fixed timestep size and a matrix T 
%   containing 'nt' number of torque curves where each curve is shifted 
%   'n*dt' in time. n is the number of duplicated torque curves. The sum
%   of the torque is located in column 'n+3'. The 'target' variable is
%   the maximum error in percent of the timestep size between 'dt' and 
%   shift*timestep'. 'shift' is number of timesteps in 'dt' found by
%   rounding 'dt/timestep'. This rounding produces the error.
%
%------ INPUT -----------------------------------------------------------
source = MD6;       %Source: torque in column 2 and time in column 3
target = 0.001;     %Error target [%]
nt = 10;            %Number of torque curves (n+1)                       
g = 9.82;           %Gravity               
r = 250;            %Runner radius [mm]
Z = 22;             %Number of buckets
H = 75;             %Head [m]
%------ INPUT - END -----------------------------------------------------

c1 = sqrt(2*g*H);           %Bucket inlet velocity
omega = (c1/(2*r))*1000;    %Angular velocity [rad/s]
fz = (omega*Z)/(2*pi);      %Bucket frequency [1/s]
dt = 1/fz;                  %Bucket period [s]

%Create temporary torque matrix including (t,T) = (time,torque) = (0,0)
T_temp(1,1) = 0;
T_temp(2:size(source,1)+1,1) = source(:,3);
T_temp(1,2) = 0;
T_temp(2:size(source,1)+1,2) = source(:,2);
clear source

%Find minimum timestep in T_temp
for i = 2:size(T_temp(:,1),1)
    
    T_temp(i,3) = T_temp(i,1)-T_temp(i-1,1);
    
end
min_step = min(T_temp(2:end,3));

%Total time
t_tot = T_temp(end,1);

%Inital values for timestep iteration
error = 1;
timestep = 1;  
i = size(T_temp(:,1),1);

%Find a timestep that meets the required target
while error >= target           
    
    timestep = t_tot/i;
    
    %Number of timesteps in one period
    shift = round(dt/timestep);
    
    %Error
    error = abs((dt-timestep*shift)/timestep);
    %error = abs((dt-timestep*shift)/dt);
    
    if timestep > 0.5*min_step
        error = 1; 
    end
    
    if error >= target
        i = i + 1;
    else
        break
    end

end          

%Create timeline in torque matrix T
T(:,1) = 0:timestep:(t_tot+(nt*dt));

%Create spline of the original torque source
Ts(:,1) = 0:timestep:T_temp(end,1);
Ts(:,2) = spline(T_temp(:,1),T_temp(:,2),Ts(:,1));

%Insert the torque series in T matrix
L = size(Ts,1);
for k = 1:nt
   
   if k == 1
       T(1:L,k+1) = Ts(:,2);
   else
       T(shift*(k-1):shift*(k-1)+L-1,k+1) = Ts(:,2);
   end
    
end

%Create total torque sum in column n+3
p = find(T(:,k+1)~=0);
p = p(end);      %Last row number in T matrix

for g = 1:p
   
    T(g,k+2) = sum(T(g,2:k+1));
    
end

%Delete all rows (empty) below row p 
T(p+1:size(T,1),:) = [];

%Delete unwanted variables from the workspace
clear Ts k nt p L g T_temp

%------ END -------------------------------------------------------------